###############################################
#
#  WALS data implementation in R
#
#
#  Author: Hans-Joerg Bibiko
#
#    mail: bibiko@eva.mpg.de
#
#    date: 16/08/2006
#
#
#    load source via: source("PATH_TO_WALS.R/wals.r", chdir=TRUE)
#
#
#  required packages: maps, mapdata
#
###############################################

#init global data
mycol<-c("black","red","green3","blue","cyan","magenta","yellow","gray","lightgreen","lightblue","lightgray","violet","pink","darkblue")

#####
#
# How to import the data from files
#
#lgs<-read.table('wals_lgs.tab', row.names=1, dec=",",  sep="\t", quote="", col.names=c("walscode","name","long","lat","genus","family","macroarea"), colClasses=c("character","character","numeric","numeric","character","character","character"))
#mat<-read.table('./Desktop/WALS_R/wals_matrix.tab', row.names=1, sep="\t")
#mat<-as.matrix(mat)
#colnames(mat)<-1:ncol(mat)
#
#featNames<-read.table('features.tab', sep="\t", quote="", col.names=c("title","area"), colClasses=c("character","character"))
#featValues<-read.table('featValue.tab', sep="\t", comment.char = "", quote="", col.names=c("feat","value","name","shape","color"), colClasses=c("numeric","numeric","character","numeric","character"))
#featNamesX<-read.table('featuresX.tab', sep="\t", quote="", col.names=c("title","area"), colClasses=c("character","character"))
#featNamesX <- cbind(featNamesX, read.table('./Desktop/WALS_R/gradient.tab', colClasses=c("logical")))
#colnames(featNamesX) <- c("title","area","gradient")
#
#featValuesX<-read.table('./Desktop/WALS_R/featValueX.tab', sep="\t", fill=TRUE, comment.char = "", quote="", col.names=c("feat","value","name","shape","color"), colClasses=c("numeric","numeric","character","numeric","character"))
#featValuesX[1:nrow(featValuesX),"shape"]<-21
#featValuesX[1:nrow(featValuesX),"color"]<-mycol[featValuesX$value+1]
#
#matX <- matrix(data=NA, nrow=nrow(mat), ncol=nrow(featNamesX), dimnames=list(rownames(mat),1:nrow(featNamesX)))
#rownames(matX) <- rownames(mat)
#
#linkX<-read.table('./Desktop/WALS_R/x_feat_X.tab', sep="\t", quote="", colClasses=c("numeric","numeric","numeric","numeric"), skip=1)
#colnames(linkX) <- c("featID","value","featXID","valueX")
#for(i in 1:nrow(linkX)) {
#	matX[listCodeForFeature(linkX[i,1],linkX[i,2]),linkX[i,3]]<-linkX[i,4]
#}
#for(i in 1:nrow(linkX)) {  
#	featNamesX[linkX[i,3],"area"] <- featNames[linkX[i,1],"area"]
#}
#
# save(mat,lgs,featNames,featNamesX,featValues,featValuesX,matX,linkX,file='******/wals.Rdata')
#
#####

###############################################
#
#   main data structure
#
# lgs         := named rows via language code (WALS code)
#             := named columns: walscode, name, long, lat, genus, family, macroarea     
#
# mat         := named rows via language code (WALS code)
#             := column number represents the feature code (for WALS 1 to 142)
#
# featNames   := row number represents the feature code
#             := named columns: title, area
#
# featValues  := row number only a counter
#             := named columns: feat, value, name, shape, color
#                    feat   :=  feature number as numeric
#                    value  :=  defined value as numeric within feat ()
#                    name   :=  short description of the value
#                    shape  :=  symbol shape as numeric, look for 'pch' in ?par
#                    color  :=  "#FF00AA" hex color code within leading '#' as character vector or other color definitions - see ?colors()
#
# X means Extended version, done by Michael Cysouw and Mihai Albu := delete logical dependencies
#
# matX        := named rows via language code (WALS code)
#             := column number represents the feature code (for WALS 1 to 257)
#
# featNamesX  := row number represents the feature code
#             := named columns: title, area
#
# featValuesX := row number only a counter
#             := named columns: feat, value, name, shape, color
#                    feat   :=  feature number as numeric
#                    value  :=  defined value as numeric within feat ()
#                    name   :=  short description of the value
#                    shape  :=  symbol shape as numeric, look for 'pch' in ?par; all set to 21 (circle)
#                    color  :=  set automatically by using mycol
#
#
# link table WALS to WALSX
# linkX       := row number only a counter
#             := named columns: featID, value, featXID, valueX
#
#
# These tables are stored in the file 'wals.Rdata'. If you add or edit data you have to save these tables with the command:
#
#    save(mat,lgs,featNames,featNamesX,featValues,featValuesX,matX,linkX,file='PATH_TO_FILE/wals.Rdata')
#
#
###############################################

###############################################
#
#   EXAMPLE COMMANDS
#
# featValues[featValues$feat == 1 & featValues$value == 2,] => returns details about value 2 for feature 1
#
# lgs["ger","lat"]  => Latitude for German(Standard)
#
# lgs[lgs[,"family"]  == "Mixe-Zoque",] => returns the subset of lgs for all languages which are defined within the family 'Mixe-Zoque'
#
# lgs[lgs[,"family"]  == "Mixe-Zoque","name"] => returns a character vector with all language names for family 'Mixe-Zoque'
#
# rownames(lgs[lgs[,"name"]  == "Hadza",])   What is the WALS code for Hadza
#
# mat["ger",9]  => returns the value for feature 9 of WALS code ger
#
# mat[rownames(lgs[lgs[,"name"]  == "Hadza",]),1]  => returns the short description of the value for feature 1 of Hadza
#
#
###
# lg  <- c("ger","aar")
# col <- c("pink","blue")
# shp <- c(21, 23)
# showLang(lg, shape=shp, color=col, mapColor=c("lightblue","lightgreen"), cex=3, mapMar=c(-10,10))
###
#
# showFamily("Altaic", genus=TRUE, regions=c("China","Mongolia"), cex=2.5, legend=TRUE)
#
# showMap(66, regions="China", cex=3, legend=FALSE)
# showMapX(66, cex=3)
#
#
# showMap(24,m="matX",interior=F,valOrder=c(1,2),center=c(135,-4),mapMar=c(-10,10),zoom=T)
#      displays feature 24 of WALSX, without boundaries(interior=F), view area center 135°W 4°S extra margin width 10°
# showMap(24,m="matX",center=c(135,-4),mapMar=c(-2,2),zoom=T,label=TRUE,label.type="name",textD=-.2,leg.pos="bottom")
#      dito but with names under the dots and a smaller area 
# showMap(24,m="matX",center=c(135,-4),mapMar=c(-2,2),zoom=T,label=TRUE,textD=.15,leg.pos="bottom",mapColor=c("lightblue","lightgreen"))
#
###
#
# # the next lines plots the WALS map 85 value 1 and 4 (1 over 4) into a pdf file 'map85.pdf' in current path:
# pdf(file='map85.pdf')
# showMap(85, add=TRUE, valOrder=c(1,4))
# dev.off()
#
# # to save a map for later editting (e.g. with Adobe Illustrator) use this
# ps.options(horizontal=FALSE)
# postscript(file='map.ps')
# showMap(1, add=TRUE)
# dev.off()
#
# 
#
#
# # if you have installed RSvgDevice you also can generate a SVG file
### 
#
###############################################


#init global data
load('wals.Rdata')
coldelta<-1 # for automatic coloring


#---------------------------------------------------
showLang <- function(lg, zoom=TRUE, center=NULL, regions=NULL, add=FALSE, label=NULL, shape=21, color="red", mapMar=c(-1,1), textD=0, mapColor=NULL, ...) {
	
	# lg         => a character vector of language codes; e.g. c("ger","aar")
	# shape/color=> whether a single parameter for all lgs or according to the length of lg two vectors of shape and colors; for shape see pch in ?par
	# mapColor   = c("Background_Color", "Landmass_Color"); e.g. c("lightblue","lightgreen")
	# mapMar     := extra margin in degrees, mapMar[1] < mapMar[2]!
	# zoom=FALSE => the whole world will be displayed
	# textD      => labels are printed centered for long and lat;
	#        with textD you can specify the amount in DEGREES to place the label over or under the symbol
	# regions    => are specified within the package 'maps'; e.g. c("China","Mongolia")
	#
	# cex        => scaling factor for symbol size
	# lwd        => line width for drawing the map's coastlines and symbol borders
	#
	# with center and mapMar you can specify the displayed area !zoom have to be TRUE!
	#
	
	options(warn=-1)
	
	#init legend text
	leg<-c(list(0,0,"init"))
	
	#init some variables	
	long<-c()
	lat<-c()
	xRange=NULL
	yRange=NULL

	#set the zoom area
	if(length(lg) == 1) {
		long <- lgs[lg, "long"] + mapMar
		lat <- lgs[lg, "lat"] + mapMar
	} else {
		long <- range(lgs[lg, "long"])
		lat <- range(lgs[lg, "lat"])
	}

	#set the center of the map according to center and mapMar
	if(!is.null(center)){
		long <- center[1] + mapMar
		lat <- center[2] + mapMar
	}

	#check for unkown language
	if(is.na(long[1])) {
		stop("Unknown language code detected!")
	}

	#if add open a new window 
	if(!add) {
		do.call(.Options$device,list(pointsize=10))
#		if(.Platform$OS.type == "windows")
#			windows(pointsize=10)
#		else
#			quartz(pointsize = 10)
	}

	#if regions are set rearrange zoom area; regions are defined within package 'maps'
	if (is.null(regions)) {
		if(zoom & !is.null(long)) {
			xRange <- range(long) + mapMar
			yRange <- range(lat) + mapMar
		} else {
			xRange <- c(-180,180)
			yRange <- c(-90,90)
			long <- xRange
			lat <- yRange
		}
	}

	#check for libraries maps and mapdata
	if(library(maps, logical.return=TRUE)) {
		library(mapdata)
		#set background and landmass colors
		if(is.null(mapColor))
			map("worldHires", regions, xlim= xRange, ylim= yRange,  ...)
		else
			map("worldHires", regions, xlim= xRange, ylim= yRange,  bg=mapColor[1], col=mapColor[2], fill=TRUE, ...)
		#draw box with long/lat	
		map.axes()
	} else {
		#only symbols will be plotted without coastlines
		if(!is.null(long)) {
			warnings("The packages 'maps' and 'mapdata' are required. Plot only dots without coastlines.")
			plot(long,lat,type="n")
		}
	}
	
	#if regions show only symbols which are inside of these regions
	if (!is.null(regions)) {
		m = map("worldHires",regions,plot=FALSE,fill=TRUE)
		min<-map.where(m,lgs[lg,"long"],lgs[lg,"lat"])
	}

	#if shape and/or color is not a vector, create a vector of equal values 
	if(length(shape) == 1) shape<-rep(shape,length(lg))
	if(length(color) == 1) color<-rep(color,length(lg))

	#draw symbols on the map
	points(lgs[lg[!is.na(min)],"long"], lgs[lg[!is.na(min)],"lat"], pch=shape[!is.na(min)], col="black", bg=color[!is.na(min)], ...)
	
	#if label draw the code names
	if(!is.null(label)) text(lgs[lg[!is.na(min)],"long"], lgs[lg[!is.na(min)],"lat"]+textD,label)
}

#---------------------------------------------------
showFamily <- function(fam, famcol=NULL, zoom=TRUE, genus=FALSE, walscode=FALSE, add=FALSE, textD=-1, legend=FALSE, regions=NULL, mapMar=c(-1,1), ...) {
	
	#showFamily(c("Altaic","Sino-Tibetan"),genus=TRUE,regions=c("Mongolia","China"),cex=2,legend=TRUE)
	
	if(!add)  {
		do.call(.Options$device,list(pointsize=10))
#		if(.Platform$OS.type == "windows")
#			windows(pointsize=10)
#		else
#			quartz(pointsize = 10)
	}

	
	options(warn=-1)
	
	leg<-c(list(0,0,"init"))
	curcol<-0
	long<-c()
	lat<-c()
	xRange=NULL
	yRange=NULL
	for(i in 1:length(fam)) {
		if(length(lgs[lgs[,"family"]  == fam[i],"lat"])>0) {
			long <- c(long,range(lgs[lgs[,"family"] == fam[i], "long"]))
			lat <- c(lat,range(lgs[lgs[,"family"] == fam[i], "lat"]))
		}
	}
	if (is.null(regions)) {
		if(zoom & !is.null(long)) {
			xRange <- range(long) + mapMar
			yRange <- range(lat) + mapMar
		} else {
			xRange <- c(-180,180)
			yRange <- c(-90,90)
			long <- xRange
			lat <- yRange
		}
	}
	if(library(maps, logical.return=TRUE)) {
		library(mapdata)
		map("worldHires", regions, xlim= xRange, ylim= yRange, ...)
		map.axes()
	} else {
		if(!is.null(long)) {
			warnings("The packages 'maps' and 'mapdata' are required. Plot only dots without coastlines.")
			plot(long,lat,type="n")
		}
	}
	if (!is.null(regions)) m = map("worldHires",regions,plot=FALSE,fill=TRUE)
	if((genus & length(listGenera(fam))>length(mycol)) | length(fam)>length(mycol)) {
		if(genus) {
			mycol<-colors()
			coldelta<-600/length(listGenera(fam))
		} else {
			mycol<-colors()
			coldelta<-600/length(fam)
		}
	}
	for(i in 1:length(fam)) {
		if(dim(lgs[lgs[,"family"]  == fam[i],])[1]>0) {
			if(genus) {
				g<-listGenera(fam[i])
				leg<-c(leg,list(1000, 1, fam[i]))
				for(j in 1:length(g)) {
					curcol<-curcol+coldelta
					long <- lgs[lgs[,"family"]  == fam[i] & lgs[,"genus"]  == g[j], "long"]
					lat <- lgs[lgs[,"family"]  == fam[i] & lgs[,"genus"]  == g[j], "lat"]
					if (!is.null(regions)) {
						min<-map.where(m,long,lat)
						points(long[!is.na(min)], lat[!is.na(min)], pch=21, col="black", bg=mycol[curcol], ...)
					} else points(long, lat, pch=21, col="black", bg=mycol[curcol], ...)
					
					leg<-c(leg,list(21, mycol[curcol], g[j]))
					if(walscode) {
						wcode <- rownames(lgs[lgs[,"family"]  == fam[i] & lgs[,"genus"]  == g[j], ])
						text(long, lat+textD, wcode)
					}
				}
			} else {
				long <- lgs[lgs[,"family"]  == fam[i],"long"]
				lat <- lgs[lgs[,"family"]  == fam[i],"lat"]
				if(is.null(famcol)) {
					curcol<-curcol+coldelta
					if (!is.null(regions)) {
						min<-map.where(m,long,lat)
						points(long[!is.na(min)], lat[!is.na(min)], pch=21, col="black", bg=mycol[curcol], ...)
					} else points(long, lat, pch=21, col="black", bg=mycol[curcol], ...)
					leg<-c(leg,list(21,col=mycol[curcol],fam[i]))
				} else if(length(famcol)==1) {
					if (!is.null(regions)) {
						min<-map.where(m,long,lat)
						points(long[!is.na(min)], lat[!is.na(min)], pch=21, col="black", bg=famcol, ...)
					} else points(long, lat, pch=21, col="black", bg=famcol, ...)
						leg<-c(leg,list(21,famcol,fam[i]))
					} else {
					if (!is.null(regions)) {
						min<-map.where(m,long,lat)
						points(long[!is.na(min)], lat[!is.na(min)], pch=21, col="black", bg=famcol[i], ...)
					} else points(long, lat, pch=21, col="black", bg=famcol[i], ...)
						leg<-c(leg,list(21,famcol[i],fam[i]))
					}
				if(walscode) {
					wcode <- rownames(lgs[lgs[,"family"]  == fam[i], ])
					text(long, lat+textD, wcode)
				}
			}
		} else { 
			warning(cat(fam[i],"is unknown"))
		}
	}
	palette("default")
	if(legend && length(leg)>3) plotLegend(leg)
	options(warn=0)
	
}

#---------------------------------------------------
plotLegend <- function(leg, cex=1.4, ...) {
	
	# leg should be a list(symbol,color,label);
	#       symbol: pch (see ?par)
	#       color : a) "#FF00AA" hexcode, b) "red", c) 1 (according to palette()
	#       label : any string
	#
	# first item is a dummy and won't draw
	# if symbol == 1000 only the label will be plotted (for a kind of header) 
	
	#create a new window
	#if(.Platform$OS.type == "windows")
	#	windows(pointsize=10)
	#else
	#	quartz(pointsize = 10)
	do.call(.Options$device,list(pointsize=10))
	plot.new()
	plot.window(xlim=c(0,20),ylim=c(0,40))
	
	n=4 # start at leg[4]
	c=1
	csp=0 # space between columns
	while (n < length(leg)) {
		#is item a 'header'
		if (leg[[n]][1] == 1000) {
			text(csp-.5,(40-c*2), leg[(n+2)], pos=4)
		} else {
			points(csp,(40-c*2), pch=leg[[n]][1], col="black", bg=leg[[n+1]][1], cex=cex, pos=4)
			text(csp+1,(40-c*2), leg[(n+2)], pos=4)
		}
		n=n+3 # three columns per record
		c=c+1 # next row
		
		#20 items a column
		if(c>20) {
			c=1
			csp=csp+8.5
		}
	}
}

#---------------------------------------------------
listGenera <- function(fam = NULL, codes = NULL) {
	# lists all genera or all genera for a family/families
	# lists all genera for codes; codes has priority for fam!
	g <- c()
	if (is.null(codes)) {
		if (is.null(fam)) {
			g <- sort(unique(lgs[,"genus"]))
		} else {
			for(i in 1:length(fam))	{
				if(dim(lgs[lgs[,"family"]  == fam[i],])[1]>0) {
					g <- c(g,sort(unique(lgs[lgs[,"family"]  == fam,"genus"])))
				}
			}
		}
	} else {
		g <- sort(unique(lgs[codes,"genus"]))
	}
	return(g)
}

#---------------------------------------------------
listFamilies <- function(codes = NULL) {
	#lists all families or for codes
	if (is.null(codes)) {
		g <- sort(unique(lgs[,"family"]))
	} else {
		g <- sort(unique(lgs[codes,"family"]))
	}
	return(g)
}

#---------------------------------------------------
listMacroareas <- function(codes = NULL) {
	#lists all macro areas or for codes
	if (is.null(codes)) {
		g <- sort(unique(lgs[,"macroarea"]))
	} else {
		g <- sort(unique(lgs[codes,"macroarea"]))
	}
	return(g)
}

#---------------------------------------------------
countDataPoints <- function(codes = NULL, bestOf = NULL,  cntBest = 1, m = "mat") {
	# if codes = NULL then take all languages of lgs
	####
	# examples:
	# get the 10 best described languages: countDataPoints(bestOf="list",cntBest=10)
	# get the best lg of each family: countDataPoints(bestOf="families")
	# get the two best of each family for specific lgs: countDataPoints(c("had","ger","eng","rus"),bestOf="families",cntBest=2)

	if (is.null(codes)) codes <- rownames(lgs)
	if (length(codes)==1) { # only for convenience
		codes = c(codes,codes)
		bestOf="list"
		cntBest=1
	}
	if (is.null(bestOf)) {
		apply(get(m)[codes,],1,function(x) {sum(!is.na(x))})
	} else {
		switch(bestOf,
			list = {
				mm <- as.matrix(apply(get(m)[codes,],1,function(x) {sum(!is.na(x))}))
				mm[order(mm,decreasing=TRUE),][1:min(cntBest,length(mm))]
			},
			genera = {
				g <- c()
				gen <- listGenera(codes=codes)
				for (i in gen) {
					a <- rownames(lgs[lgs[,"genus"]==i,])
					g <- c(g, countDataPoints(a[a %in% codes],bestOf="list",cntBest=cntBest))
				}
				return(g)
			},
			families = {
				g <- c()
				gen <- listFamilies(codes=codes)
				for (i in gen) {
					a <- rownames(lgs[lgs[,"family"]==i,])
					g <- c(g, countDataPoints(a[a %in% codes],bestOf="list",cntBest=cntBest))
				}
				return(g)
			}
		)
			
	}
}

#---------------------------------------------------
dist.rel.hamm <- function(lg_codes) {
	l<-length(lg_codes)
	m<-matrix(0,nrow=l,ncol=l,dimnames=list(lg_codes, lg_codes))
	for (i in 1:l)	{
		for (j in 1:(i-1)) {
			sim<-sum(na.omit(mat[lg_codes[i],]==mat[lg_codes[j],]))
			diff<-sum(na.omit(mat[lg_codes[i],]!=mat[lg_codes[j],]))
			m[j,i]<-m[i,j]<-diff/(sim+diff)
		}
	}
	return(m)
}

#---------------------------------------------------
codeToName <- function(walscode) lgs[walscode,"name"]

#---------------------------------------------------
nameToCode <- function(name) rownames(lgs[lgs[,"name"] == name,])

#---------------------------------------------------
listCodeForFeature <- function(feat, val=NULL, m="mat", lgset=NULL) {
	
	#
	# Examples:  codeToName(listCodeForFeature(2,1))
	#              returns all lgs names which are defined for feature 2 having the value 1
	#
	#            listCodeForFeature(2)
	#              returns all lgs wals codes defined within feature 2
	#
	#            listCodeForFeature(c(1,142))
	#              returns all attested lgs within feature 1 AND 142
	#
	#            listCodeForFeature(c(1,142), lgs=c("alb", "ava", "zsq"))
	#              returns "alb","ava" because "alb","ava" are attested for feature 1 and 142, "zsq" is only attested for feature 142
	#
	#            listCodeForFeature(c(1,142),c(3,2))
	#              returns 15 lgs which have for feature 1 value 3 AND for feature 142 value 2
	#
	# parameter m := specify the name of the used matrix, default is the original WALS matrix 'mat'
	#    for using WALSX call: listCodeForFeature(feat, val, m="matX")
	#
	
	if(length(feat) == 1) {
		out <- rownames(get(m)[!is.na(get(m)[,feat]),1:2])
		if(!is.null(val)) {
			v <- rownames(get(m)[get(m)[,feat]==val ,1:2])
			out <- out[out %in% v]
		}
	} else {
		if(is.null(val)) {
			b <- T
			for(i in feat) b <- b & !is.na(get(m)[,i])
			out <- rownames(get(m)[b,1:2])
		} else {
			if(length(feat) != length(val)) stop("Vector feat and val must have the same length!")
			out <- listCodeForFeature(feat)
			for(i in 1:length(feat)) {
				if(val[i] > countFeatureValues(feat[i],m=m)) stop(paste("Value",val[i],"for feature",feat[i],"is not defined!"))
				out <- out[out %in% listCodeForFeature(feat[i],val[i],lgset=out)]
			}
		}
	}
	ifelse(is.null(lgset), return(out), return(lgset[lgset %in% out]))
}


#---------------------------------------------------
countFeatureValues <- function(feat, m="mat") max(get(m)[!is.na(get(m)[,feat]),feat])
	#
	# parameter m := specify the name of the used matrix, default is the original WALS matrix 'mat'
	#    for using WALSX call: countFeatureValues(feat, m="matX")
	#

#---------------------------------------------------
showMap <- function(feat, leg=TRUE, leg.cex=0.9, leg.pos="bottomleft", showtitle=TRUE, valOrder=NULL, m="mat", cex=2, 
			regions=NULL, add=TRUE, label=FALSE, label.type="code", zoom=FALSE, ...)
{
	
	#
	# valOrder:
	# valOrder = c(3,2) prints only value 3 over value 2
	# valOrder = "asc"  prints most common at the top
	# valOrder = "desc" prints most common at the bottom
	#
	# use add=TRUE if you want plot into the same window or if you want to plot into a device like pdf(), ps(), RSvgDevice, ...
	#
	# cex := symbol size, see ?par
	#
	# label.type := "code" or "name"
	#
	# parameter m := specify the name of the used matrix, default is the original WALS matrix 'mat'
	#    for using WALSX call: showMap(..., m="matX")
	#
	
	
	#set the used feature table according to m ('mat' or 'matX')
	fV <- "featValues"
	if(m == "matX") fV <- "featValuesX"
	
	#how many values for feature feat
	value_cnt <- countFeatureValues(feat, m=m)
	
	#init some stuff
	lg<-NULL
	shape_t<-NULL
	color_t<-NULL
	legtext<-NULL

	#if no valOrder then use the predefined order 1 to value_cnt
	if(is.null(valOrder)) {
		valOrder =c(1:value_cnt)
	} else {
		#if valOrder == "asc" or "desc"
		if(is.character(valOrder)) {
			vc<-NULL
			for(i in 1:value_cnt) vc<-c(vc,length(listCodeForFeature(feat, i, m=m)))
			vc <- order(vc)
			if(valOrder == "desc") vc <- rev(vc)
			valOrder <- vc
		} else {
		#otherwise reverse the order i.e. first valOrder item is on the top the map
			valOrder <- rev(valOrder)
		}
	}

	# preset the value descriptions albeit the valOrder with number of languages per value
	if(leg) {
		for(i in 1:value_cnt) {
			lg_ <- listCodeForFeature(feat, i, m=m)
			#if regions show only symbols which are inside of these regions
			if (!is.null(regions)) {
				mp = map("worldHires",regions,plot=FALSE,fill=TRUE)
				min<-map.where(mp,lgs[lg_,"long"],lgs[lg_,"lat"])
				legtext <- c(legtext, paste(get(fV)[get(fV)$feat == feat & get(fV)$value == i, "name"], " [",length(lg_[!is.na(min)]),"]",sep=""))
			} else {
				legtext <- c(legtext, paste(get(fV)[get(fV)$feat == feat & get(fV)$value == i, "name"], " [",length(lg_),"]",sep=""))
			}
		}
	}

	#set lg, shape, color according to valOrder
	for(i in valOrder) {
		lg_ <- listCodeForFeature(feat, i, m=m)
		lg<-c(lg, lg_)
		shape_t<-c(shape_t,rep(get(fV)[get(fV)$feat == feat & get(fV)$value == i, "shape"],length(lg_)))
		color_t<-c(color_t,rep(get(fV)[get(fV)$feat == feat & get(fV)$value == i, "color"],length(lg_)))
	}

	#print symbols
	if(label) {
		if(label.type=="code")
			label <- lg
		else
			label <- codeToName(lg)
	} else {
		label <- NULL
	}
	
	showLang(lg, shape=shape_t, color=color_t, regions=regions, add=add, label=label, cex=cex, zoom=zoom, ...)
	
	#print title?
	if(showtitle) {
		t <- "WALS"
		if(m=="matX") 
			t <- paste (t, "X feature ", feat, "\n", featNamesX[feat,"title"], sep="")
		else
			t <- paste (t, " feature ", feat, "\n", featNames[feat,"title"], sep="")
		title(t)
	}

	#print legend if leg
	if(leg) {
		leg_<-list(1,1,"init")
		for(i in valOrder) {
			leg_<-c(leg_,list(get(fV)[get(fV)$feat == feat & get(fV)$value == i, "shape"],col=get(fV)[get(fV)$feat == feat & get(fV)$value == i, "color"],legtext[i]))
		}
		#plotLegend(leg_, cex=cex)
		inleg <- as.vector(unlist(leg_))
		if(length(inleg)>3) inleg <- inleg[4:length(inleg)]
		inleg <- matrix(inleg, ncol=3, byrow=T)
		
		if(is.character(leg.pos)) {
			legend(x=leg.pos,legend=inleg[,3],pt.bg=inleg[,2],pch=as.numeric(inleg[,1]),cex=leg.cex)
		} else {
			legend(x=leg.pos[1],y=leg.pos[2],legend=inleg[,3],pt.bg=inleg[,2],pch=as.numeric(inleg[,1]),cex=leg.cex)
		}
	}
}

#---------------------------------------------------
tkWALS <- function(m="mat")
{
	library(tcltk)
	require(tcltk) || stop("tcltk support is absent")
	require(graphics); require(stats)
	
	mx <- tclVar(0)
	my <- tclVar(0)
	zl <- tclVar(50)
	mv <- NULL
	
	remove <- function(mv) {
		if(is.numeric(mv) & as.numeric(tclObj(zl)) != 50) {
			z <- as.numeric(tclObj(zl))
			myy <- as.numeric(tclObj(my))
			mxx <- as.numeric(tclObj(mx))
			if(mv==1001) {
				myy <- myy+1
				tclObj(my) <- myy
			}
			if(mv==1002) {
				myy <- myy-1
				tclObj(my) <- myy
			}
			if(mv==1003) {
				mxx <- mxx+1
				tclObj(mx) <- mxx
			}
			if(mv==1004) {
				mxx <- mxx-1
				tclObj(mx) <- mxx
			}
			showMap(as.numeric(tkcurselection(ml))+1, center=c(as.numeric(tclObj(mx)),as.numeric(tclObj(my))), mapMar=c(-1*z,z),zoom=T)
		}
	} 
	
	rezoom <- function(...)
	{
		z <- as.numeric(tclObj(zl))
		if(z == 50)
			showMap(as.numeric(tkcurselection(ml))+1)
		else
			showMap(as.numeric(tkcurselection(ml))+1, center=c(as.numeric(tclObj(mx)),as.numeric(tclObj(my))), mapMar=c(-1*z,z),zoom=T)
	}
	
	remap <- function(map)
	{
		z <- as.numeric(tclObj(zl))
		if(z==50)
			showMap(as.numeric(map)+1)
		else
			showMap(as.numeric(map)+1, center=c(as.numeric(tclObj(mx)),as.numeric(tclObj(my))), mapMar=c(-1*z,z),zoom=T)
	}
	
	base <- tktoplevel(bg="white")
    tkwm.title(base, "tkWALS")
    #tkwm.minsize(base, 200, 200)
    
    spec.frm <- tkframe(base, borderwidth=2, bg="white")
	left.frm <- tkframe(spec.frm, borderwidth=20, bg="white")
	right.frm <- tkframe(spec.frm, borderwidth=20, bg="white")
	
	frame1 <- tkframe(left.frm, relief="flat", borderwidth=3, bg="white")
	frame2 <- tkframe(left.frm, relief="flat", borderwidth=3, bg="white")
	frame3 <- tkframe(right.frm, relief="flat", borderwidth=3, bg="white")
	
	tkpack(tklabel(frame1, text="Move", bg="white"))
	tkpack(frame1, tkbutton(frame1,text="^", command=function()remove(mv=1001)), side="top")
	tkpack(frame1, tkbutton(frame1,text="v", command=function()remove(mv=1002)), side="bottom")
	tkpack(frame1, tkbutton(frame1,text=">", command=function()remove(mv=1003)), side="right")
	tkpack(frame1, tkbutton(frame1,text="<", command=function()remove(mv=1004)), side="left")

	tkpack(tklabel(frame2, text="Zoom", bg="white"))
	tkpack(tkscale(frame2, command=rezoom, from=2, to=50,showvalue=T, variable=zl, resolution=0.5, orient="horiz", bg="white"))

	tkpack(tklabel(frame3, text="Maps", bg="white"))
	ml <- tklistbox(frame3, height=9, width=80, bg="white", exportselection="0", selectmode="browse")
	tkpack(frame3, ml, side="top")
	ml.yscroll <- tkscrollbar(frame3)
	tkconfigure(ml,yscrollcommand=paste(.Tk.ID(ml.yscroll),"set")) 
	tkconfigure(ml.yscroll,command=paste(.Tk.ID(ml),"yview"))
	tkpack(ml.yscroll, ml,side="left",fill="y")
	if(m=="mat") {
		for(i in 1:nrow(featNames)) tkinsert(ml,"end", paste(i,"-",featNames[i,"title"]))
	}
	
	tkbind(ml, "<Double-Button-1>", function()remap(tkcurselection(ml)))
	
	tkpack(left.frm, side="right")
	tkpack(right.frm, side="left")
	tkpack(frame1, side="top")
	tkpack(frame2, side="bottom")
	tkpack(frame3, side="top")
	
	tkpack(spec.frm, tkbutton(base,text="Quit", command=function()tkdestroy(base)))
	
}


#---------------------------------------------------
genTreeForCodes <- function(lg, type="analysis", separ=" | ", showLg=FALSE, showGen=TRUE, feat=NULL, m="mat") {
	#only a draft
	out <- NULL
	allFam<-unique(lgs[lg,"family"])
	allFam<-allFam[order(allFam)]
	if(type=="tree") {
		for(f in allFam) {
			out <- paste(out,f," [",1,"]","\n",sep="")
			allGen<-lgs[lg[lgs[lg,"family"] == f],"genus"]
			allGen<-unique(allGen)
			allGen<-allGen[order(allGen)]
			for(g in allGen) {
				out <- paste(out,separ,g,"\n",sep="")
				l<-lgs[lg[lgs[lg,"genus"] == g],"name"]
				out <- paste(out,separ,separ,sep="")
				ol <- paste(l,sep=", ")
				out <- paste(out,ol,"\n",sep="")
			}
		}
	}
	if(type=="analysis") {
		out <- paste(out,"languages: ", length(lg),"\n", sep="")
		out <- paste(out,"families: ", length(allFam), "\n", sep="")
		out <- paste(out,"genera: ", length(unique(lgs[lg,"genus"])), "\n", sep="")
		for(f in allFam) {
			out <- paste(out,f," (",sep="")
			out <- paste(out,length(lgs[lg[lgs[lg,"family"] == f],"genus"]), sep="")
			if(!is.null(feat)) {
				al <- listCodeForFeature(feat,m=m)
				out <- paste(out,"/", length(lgs[al[lgs[al,"family"] == f],"genus"]), "/", nrow(lgs[lgs[,"family"] == f,]), sep="")
			}
			out <- paste(out,")\n",sep="")
			if(showGen) {
				allGen<-lgs[lg[lgs[lg,"family"] == f],"genus"]
				allGen<-unique(allGen)
				allGen<-allGen[order(allGen)]
				for(g in allGen) {
					out <- paste(out,separ,g," (",sep="")
					l<-lgs[lg[lgs[lg,"genus"] == g],"name"]
					out <- paste(out,length(l),sep="")
					if(!is.null(feat)) out <- paste(out,"/",  length(lgs[al[lgs[al,"genus"] == g],"genus"]), "/", nrow(lgs[lgs[,"family"] == f & lgs[,"genus"] == g,])   ,sep="")
					out <- paste(out,")\n",sep="")
					if(showLg) out <- paste(out,separ,separ,paste(l,collapse=", "),"\n",sep="")
				}
			}
		}
	}
	return(out)
}

#---------------------------------------------------
linkInfoX <- function(featX)
{
	for(n in 1:length(featX)) {
		if(featX[n]<1 | featX[n]>nrow(featNamesX)) stop(paste("Feature",featX[n],"is not defined in matX!"))
		cnt <- countFeatureValues(featX[n], m="matX")
		out <- paste("WALSX feature ",featX[n],": ",featNamesX[featX[n],"title"],sep="")
		out <- paste(out,ifelse(featNamesX[featX[n],"gradient"]," (gradient values)",""),"\n",sep="")
		a <- unique(linkX[linkX[,"featXID"]==featX[n],c(1)])
		sp <- ifelse(length(a)==1," ","\n   ")
		out <- paste(out,"  based on WALS feature",sep="")
		for(k in 1:length(a)) out <- paste(out,sp,a[k]," - ",featNames[a[k],"title"],sep="")
		out <- paste(out,"\n",sep="")
		for(i in 1:cnt) {
			out <- paste(out,"  Value ",i,": ",featValuesX[featValuesX[,"feat"]==featX[n] & featValuesX[,"value"]==i,"name"]," [",length(listCodeForFeature(featX[n],i,m="matX")),"]","\n",sep="")
			a <- linkX[linkX[,"featXID"]==featX[n] & linkX[,"valueX"]==i,c(1,2)]
			for(k in 1:nrow(a)) out <- paste(out,"    ",a[k,1],".",a[k,2]," - ",featValues[featValues[,"feat"]==a[k,1] & featValues[,"value"]==a[k,2],"name"]," [",length(listCodeForFeature(a[k,1],a[k,2])),"]","\n",sep="")
		}
		cat(out)
	}
}

#---------------------------------------------------
featInfo <- function(feat, val=NULL, m="mat", ...)
{
	fV <- ifelse(m == "mat","featValues","featValuesX")
	fN <- ifelse(m == "mat","featNames","featNamesX")
	out <- NULL
	out <- paste("WALS",ifelse(m == "mat","","X")," feature ",feat," - ",get(fN)[feat,"title"],"\n",sep="")
	out <- paste(out,"total number of stated languages within this feature: ",length(listCodeForFeature(feat,m=m)),"\n\n",sep="")
	if(is.null(val)) {
		cnt <- countFeatureValues(feat, m=m)
		for(i in 1:cnt) {
			out <- paste(out,"Value ",i," - ",get(fV)[get(fV)[,"feat"]==feat & get(fV)[,"value"]==i,"name"],"\n",sep="")
			out <- paste(out,genTreeForCodes(listCodeForFeature(feat,i,m=m), feat=feat, ...),"\n",sep="")
		}
	} else {
		for(i in val) {
			out <- paste(out,"Value ",i," - ",get(fV)[get(fV)[,"feat"]==feat & get(fV)[,"value"]==i,"name"],"\n",sep="")
			out <- paste(out,genTreeForCodes(listCodeForFeature(feat,i,m=m), feat=feat, ...),"\n",sep="")
		}
	}
	return(out)
}